import mongoose, { Schema, Connection, Model } from "mongoose";
import dotenv from "dotenv";

interface ConnectionCache {
  [key: string]: Connection;
}
const connections: ConnectionCache = {};
dotenv.config();

const MainModel = <T>(
  db: string,
  modelName: string,
  schema: Schema<T>,
  collectionName: string
): Model<T> => {
  const db1_url = `${process.env.LOCAL_MONGO_URI}${db}`;
  const authOptions = {
    user: process.env.MONGO_USER,
    pass: process.env.MONGO_PASSWORD,
    authSource: process.env.MONGO_AUTH_DB || "admin",
    maxPoolSize: 50,
  };
  // Check if the connection already exists
  if (connections[db]) {
    return connections[db].model<T>(modelName, schema, collectionName);
  }

  try {
    const db1 = mongoose.createConnection(db1_url, authOptions);

    // Cache the connection
    connections[db] = db1;

    db1.on("connected", () => {
      console.log(`Connected to MongoDB database: ${db}`);
    });

    db1.on("error", (err) => {
      console.error(
        `MongoDB connection error for database ${db}:`,
        err.message
      );
    });

    return db1.model<T>(modelName, schema, collectionName);
  } catch (error) {
    console.error("Database connection error:", (error as Error).message);
    throw error;
  }
};

export default MainModel;
